@@ -6,6 +6,7 @@ import torch
 
 import pytorch_lightning as pl
 from torch import optim
+import matplotlib
 
 from deepforest import utilities
 from deepforest import dataset
@@ -13,7 +14,7 @@ from deepforest import get_data
 from deepforest import model
 from deepforest import predict
 from deepforest import evaluate as evaluate_iou
-
+from pytorch_lightning.callbacks import LearningRateMonitor
 
 class deepforest(pl.LightningModule):
     """Class for training and predicting tree crowns in RGB images
@@ -27,7 +28,7 @@ class deepforest(pl.LightningModule):
             self: a deepforest pytorch ligthning module
         """
         super().__init__()
-
+        
         # Read config file - if a config file exists in local dir use it,
         # if not use installed.
         if os.path.exists("deepforest_config.yml"):
@@ -48,7 +49,11 @@ class deepforest(pl.LightningModule):
 
         self.num_classes = num_classes
         self.create_model()
+        
         #Label encoder and decoder
+        if not len(label_dict) == num_classes:
+            raise ValueError("label_dict {} does not match requested number of classes {}, please supply a label_dict argument {'label1':0, 'label2':1, 'label3':2 ... etc} for each label in the dataset".format(label_dict, num_classes))
+        
         self.label_dict = label_dict
         self.numeric_to_label_dict = {v: k for k, v in label_dict.items()}
 
@@ -72,12 +77,18 @@ class deepforest(pl.LightningModule):
         self.model = model.create_model(self.num_classes, self.config["nms_thresh"],
                                         self.config["score_thresh"])
 
-    def create_trainer(self, logger=None, callbacks=None, **kwargs):
+    def create_trainer(self, logger=None, callbacks=[], **kwargs):
         """Create a pytorch ligthning training by reading config files
         Args:
             callbacks (list): a list of pytorch-lightning callback classes
         """
-
+        
+        #If val data is passed, monitor learning rate
+        if not self.config["validation"]["csv_file"] is None:
+            if logger is not None:
+                lr_monitor = LearningRateMonitor(logging_interval='epoch')
+                callbacks=callbacks.append(lr_monitor)
+        
         self.trainer = pl.Trainer(logger=logger,
                                   max_epochs=self.config["train"]["epochs"],
                                   gpus=self.config["gpus"],
@@ -190,7 +201,8 @@ class deepforest(pl.LightningModule):
         result = predict.predict_image(model=self.model,
                                        image=image,
                                        return_plot=return_plot,
-                                       device=self.device)
+                                       device=self.device,
+                                       iou_threshold=self.config["nms_thresh"])
         
         #Set labels to character from numeric if returning boxes df
         if not return_plot:
@@ -217,7 +229,8 @@ class deepforest(pl.LightningModule):
                                       csv_file=csv_file,
                                       root_dir=root_dir,
                                       savedir=savedir,
-                                      device=self.device)
+                                      device=self.device,
+                                      iou_threshold=self.config["nms_thresh"])
 
         #Set labels to character from numeric
         result["label"] = result.label.apply(lambda x: self.numeric_to_label_dict[x])
@@ -271,6 +284,11 @@ class deepforest(pl.LightningModule):
                                       thresh=thresh,
                                       device=self.device)
 
+        #edge case, if no boxes predictioned return None
+        if result is None:
+            print("No predictions made, returning None")
+            return None
+        
         #Set labels to character from numeric if returning boxes df
         if not return_plot:
             result["label"] = result.label.apply(lambda x: self.numeric_to_label_dict[x])
@@ -306,18 +324,12 @@ class deepforest(pl.LightningModule):
             self.log("val_{}".format(key), value, on_epoch=True)
 
         return losses
-
-    def validation_end(self, outputs):
-        avg_loss = torch.stack([x['val_loss'] for x in outputs]).mean()
-        comet_logs = {'val_loss': avg_loss}
-
-        return {'avg_val_loss': avg_loss, 'log': comet_logs}
-
+        
     def configure_optimizers(self):
-        self.optimizer = optim.SGD(self.model.parameters(),
+        optimizer = optim.SGD(self.model.parameters(),
                                    lr=self.config["train"]["lr"],
                                    momentum=0.9)
-        self.scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(self.optimizer,
+        scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(optimizer,
                                                                     mode='min',
                                                                     factor=0.1,
                                                                     patience=10,
@@ -327,7 +339,12 @@ class deepforest(pl.LightningModule):
                                                                     cooldown=0,
                                                                     min_lr=0,
                                                                     eps=1e-08)
-        return self.optimizer
+        
+        #Monitor rate is val data is used
+        if self.config["validation"]["csv_file"] is not None:
+            return {'optimizer':optimizer, 'lr_scheduler': scheduler,"monitor":'val_classification'}
+        else:
+            return optimizer
 
     def evaluate(self,
                  csv_file,
@@ -355,7 +372,10 @@ class deepforest(pl.LightningModule):
                                            csv_file=csv_file,
                                            root_dir=root_dir,
                                            savedir=savedir,
-                                           device=self.device)
+                                           device=self.device,
+                                           iou_threshold=self.config["nms_thresh"])
+        
+        predictions["label"] = predictions.label.apply(lambda x: self.numeric_to_label_dict[x])
         ground_df = pd.read_csv(csv_file)
 
         # if no arg for iou_threshold, set as config
