@@ -254,6 +254,15 @@ def check_file(df):
     return df
 
 
+def check_image(image):
+    """Check an image is three channel, channel last format
+        Args:
+           image: numpy array
+        Returns: None, throws error on assert
+    """
+    if not image.shape[2] == 3:
+        raise ValueError("image is expected have three channels, channel last format, found image with shape {}".format(image.shape))
+    
 def project_boxes(df, root_dir, transform=True):
     """
     Convert from image coordinates to geopgraphic cooridinates
